%% SEG4KING_S9_15
%{


# DESCRIPTION #
SEG4KING Prepare and light proces SEG-Y files for fast automatic loading in
Kingdom-Suite or other geological interpretation software and some more utilities.
Author: Marcel.li Farran (ICM-CSIC), 05/IV/2018 
http://http://gma.icm.csic.es/
 

# NEW IN VERSION 9 #
 
- Only for Matlab Version 8 or newer, for older Matlab use SEG4king8
- New GUI with inputsdlg.m (Copyright (c) 2015, Takeshi Ikuma)
- COMMON MIDPOINT & NMO (in-Out) (for Multichannel Adquisition). 
- X&Y recalculated  positions are stored CDP X-CDP Y fields (byte 181-185). 
	"FieldRecord" & "TraceNumber" trace headers necessary. 
- Offsets will be written in Distance Source-receiver header (byte 37-40).
- NOTCH and BANDPASS filter.
9.15 commented line  Answer.BPfilter(5)= mat2cell(false) for Matlab 2107 incompatibility;  
 
# FROM PREVIOUS VERSIONS # 

 - Group Field coordinates  as the origin of the coordinates to transform
- Repositioning of towed sources, the new positions stored in cdp X & Y.
- Replaces  the position of GPS antena by  Mid-point between source and detector 
   making easy CDP creation, sorting and staking.
-  Improved automatic signal spikes removal. 
- Change coordinate from lat-lon to UTM. (WGS84)
- Saves in SEG-Y compatible Kingdom-Suite with constant trace length.
- Allows change of UTM fuse.
- Select directory and files with the corresponding wildcard
- Interpolate if the coordinates are equal for lack of precision in the
  Decimals.
- Trace resampling.
- Filtering of spikes (sounders interferences) by entering a value of
reference. It is considered spike if for a time t the value in the trace
is larger than the adjacent ones multiplied by the reference value.
Uses RMS with Auto option
- Filter "stopped boat" traces or almost for recordings combined with bottom sampling.
- Amplitude Envelope Instance.

 
# AUXILIARY fILES & FOLDERS #

**If these files are not found, the processes related to them will be deactivated**
Must be in the Matlab Path:
Segymat folder               Always (C) 2001-2004 Thomas Mejer Hansen
inputsdlg folder or .m file  Always (C) 2015 Takeshi Ikuma
GeoUTM.m                     Always (C) Rafael Palacios, Univ. Pontificia Comillas 2006
UTMGeo.m                     Always (C) Rafael Palacios, Univ. Pontificia Comillas 2006
cvnmo.m                      For NMO(C) Seismic Processing and Imaging Group 
                                         Dept. of Physics, The Univ. of Alberta 1998
ormsby.m                     For BandPass Filter (C) E. Rietsch 2001
myhilbert.m                  For Instantaneous ampl. Envelope (C) E. Rietsch 2003
ecgTransientSuppression.m    For Notch filter.   (C) 2014, Arturo Moncada-Torres
MATLAB MAPPING TOOLBOX       If is installled a map is diplayed (not necessary)

If MATLAB MAPPING TOOLBOX exist a simple map of lines is created. 
Also KML file is created with segy files metada.

Before to run the script check that  "seg-aux" folder is in your
Matlab path (run 'pathtool' if necessary).

Sometimes is better to remove 'S4K9def.mat' file values used in the last run.   
In case of persitent errors  delete all 'S4K9def.mat' of your path folders.


# FILE NAMES #

CREATED FILES will have the same name but in a new ./KS_OK subfolder  
To the name of the resulting file a PREFIX CAN BE ADDED only if the trace data
are modifier respecting the original and are stored in the Sub folder ./KS_OK

f_      = Spikes filtered,
r_      = Resampled, 
e_      = Envelope, 
s_      = Nav. Smoothed,
o_      = Applied layback
n_      = Notch filtered
bp_     = BandPass Filtered
cdp_    = Common midpoint calculated: cdp, offsets, cdpX & cdpY headers filled. 
csk_    = brute stacked section of common mid points 
ssk_    = brute stack of channels for each shot.
+nmo_   = normal moveout calculated
-nmo_   = nomal moveout removed
Name_S# = Splitted file #: sequential number 

 The  processing sequence of the traces is:
 Notch filtered , Banpass, Envelope, Resampling, Positions transformation
(coordinates,smoothing,offsets).  


# SEGYMAT VALID SEG FILES FORMATS # 

 REV| DSF|Data Sample Format              |
 0  |  1 | 4 Byte IBM Floating Point      |        
 1  |  1 | 4 Byte IBM Floating Point      | >>>> KS. Default Input
 1  |  2 | 4 Byte two's complement Integer| 
 1  |  3 | 2 Byte two's complement Integer| 
 1  |  5 | 4 Byte IEEE FLoating Point     | >>>> ATLAS, TOPAS Parametric

###############################################################################
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
OF THE POSSIBILITY OF SUCH DAMAGE.
###############################################################################
%}

%% START

    clc;  clear all; echo off;
    SEG4KING_Version  =  'Version: 9 SEG4KING (2018)';
    color = ('g'); total = 0;  ccc = 1; warning ('off','all');  k = 0; orden = 0;
    cd (pwd);
    % test if segymat and inputsdlg exists
    while exist('SegymatVersion','file') ==  0 
        disp 'SEGYMAT not found, Install or re-start Matlab ';
        pathtool;
        break;
    end
    
    while exist('inputsdlg.m','file') ==  0
        disp 'inputsdlg.m not found, Install or set in path Matlab ';
        pathtool;
        break; 
    end
    
    ver;
    SegymatVersion;
    disp 'inputsdlg.m found';
    defaults = 'S4K9def.mat';
    
    if exist(defaults, 'file')
    disp 'Default values found in :  S4K9def.mat'
        load('S4K9def.mat');  
        Answer.DataFolder = pwd;
    else
        Answer.DataFolder = pwd;
        Answer.Survey = 'Survey';
        Answer.CoorFormat = 1;                    Answer.UTMinp = 30;
        Answer.Hemis = 1;                         Answer.UTMout = 30;
        Answer.FilesEX = 1;                       Answer.FilesFor = 1;
        Answer.Envel = 0;                         Answer.SPKfilt = 1;
        Answer.Resamp = 1;                        Answer.SmoothFactor = 0;
        Answer.Maxgap = 0;                        Answer.ONstation = 0;
        Answer.cdp = false;                       Answer.notch = 1;
        Answer.BPfilter ={10 20 500 600 false}; 
       % Answer.BPfilter(5)= mat2cell(false);  
        Answer.CHANS = 1;  
        Answer.GPSsour = 0;                       Answer.GPS1ch = 0;
        Answer.InterCH = 0;                       Answer.SoundSp = 1500;
    end             
    

%% SETTING DIALOG OPTIONS
Title  =  SEG4KING_Version;
Options.WindowStyle   =  'modal';
Options.Resize        =  'on';
Options.Interpreter   =  'tex';
Options.CancelButton  =  'on';
Options.ApplyButton   =  'on';
Options.ButtonNames   =  {'Go','Exit','Reset Defaults'}; %<- default names, included here just for illustration
Options.Sep  =  8;    % Horizontal dimension in fields
%
Prompt   =  {};
Formats  =  {};
DefAns   =  struct([]);
%
Prompt(1,:)  =  {'Prepare SEG-Y for loading in geological interpretation soft. & some more. ',[],[]};
Formats(1,1).type  =  'text';
%
Prompt(2,:)  =  {'SEG-Y''s Data Folder: ','DataFolder',[]};
Formats(2,1).type     =  'edit';
Formats(2,1).format   =  'dir';
Formats(2,1).size     =  [-1 0];
Formats(2,1).size     =  400;
Formats(2,1).span     =  [1 1];  % item is 1 field x 3 fields
DefAns(1).DataFolder  =  Answer.DataFolder;
%
Prompt(3,:)  =  {'Survey Name:      ', 'Survey',[]};
Formats(2,2).type    =  'edit';
Formats(2,2).format  =  'text';
Formats(2,2).size    =  150; % automatically assign the height
DefAns(1).Survey     =  Answer.Survey;
%  
Prompt(4,:)  =  {'COORDINATES INPUT FORMAT :','CoorFormat',' WGS84 (*most raw data)'};
Formats(3,1).type     =  'list';
Formats(3,1).style    =  'popupmenu';
Formats(3,1).items    =  {'MilliSeconds of Degree*','UTM meter','UTM centimeter','UTM decimers','UTM m in GroupX&Y','DDMMM smoothed','DDDMMMMM', 'CentiSeconds of Degree'};
DefAns(1).CoorFormat  =  Answer.CoorFormat;
%
Prompt(5,:)  =  {'Input UTM Zone:   ', 'UTMinp',' Used only for fuse change'};
Formats(3,2).type    =  'edit';
Formats(3,2).format  =  'float';
Formats(3,2).size    =  40;
Formats(3,2).limits  =  [1 60];
DefAns(1).UTMinp     =  Answer.UTMinp;
%
Prompt(6,:)         =  {'Input SEGY Data Format:             ','FilesFor','(SEG Rev.1)'};
Formats(4,1).type   =  'list';
Formats(4,1).style  =  'popupmenu';
Formats(4,1).items  =  { 'IBM4Byte:1','2Byte:3','IEEE4Byte:5'}; 
DefAns(1).FilesFor  =  Answer.FilesFor;
%
Prompt(7,:)  =  {'Output UTM Zone:', 'UTMout',''};
Formats(4,2).type    =  'edit';
Formats(4,2).format  =  'float';
Formats(4,2).size    =  40;
Formats(4,2).limits  =  [1 60]; 
DefAns(1).UTMout     =  Answer.UTMout;
%
Prompt(8,:)  =  {'SEGY Files Extension:                 ','FilesEX',[]};
Formats(5,1).type   =  'list';
Formats(5,1).style  =  'popupmenu';
Formats(5,1).items  =  {'*.seg','*.sgy','*.segy','*.tra'};
DefAns(1).FilesEX   =  Answer.FilesEX;
%
Prompt(9,:)  =  {'       Hemisphere:  ','Hemis',[]};
Formats(5,2).type   =  'list';
Formats(5,2).style  =  'popupmenu';
Formats(5,2).items  =  {'N','S'};
DefAns(1).Hemis     =  Answer.Hemis;
%
Prompt(10,:)  =  {'--------------------------------------- EXTRA Functions ---------------------------------',[],[]};
Formats(6,1).type  =  'text';Formats(6,1).size  =  [-1 0];Formats(6,1).span  =  [1 2]; 
%
Prompt(11,:)           =  {'Notch Filter                    ' ,'notch',' Bandwidth 3Hz '};
Formats(7,1).type      =  'list';
Formats(7,1).items     =  { 'NO','50 Hz','60 Hz','f1 in BandPass'};
DefAns(1).notch        =  Answer.notch;
if exist('ecgTransientSuppression','file')  ~=  2, Formats(7,1).enable  = 'off'; disp 'ecgTransientSuppression.m NOT Found'; 
else  Formats(7,1).enable  = 'on';   disp 'ecgTransientSuppression Found';
end 
%
Prompt(12,:)           =  {'CONVERT Real Signal to Inst. Ampl. Envelope ' ,'Envel',''};
Formats(7,2).type      =  'check';
Formats(7,2).unitsloc  =  'bottomleft';
if exist('myhilbert','file')   ~=  2, Formats(7,2).enable  = 'off'; disp 'myhilbert.m NOT Found' ; 
else  Formats(7,2).enable  = 'on'; disp 'myhilbert.m Found'; 
end
%
Prompt(13,:) = {'BandPass','BPfilter',' Hz (< Nyquist Frequency) '};
Formats(8,1).type = 'table';
Formats(8,1).format = {'numeric', 'numeric', 'numeric', 'numeric', 'logical'};
Formats(8,1).items = {'f1', 'f2', 'f3', 'f4', 'Apply'};
Formats(8,1).size = [ 300 40 ];
Formats(8,1).limits = {50 50 50 50 50};
Formats(8,1).span = [1 2]; 
DefAns.BPfilter =  Answer.BPfilter;
if exist('ormsby','file') ~=  2,
    Formats(8,1).enable   = 'off'; 
    Answer.BPfilter = {10 20 500 600 0};
    Answer.BPfilter(5)= mat2cell(0);
    disp 'ormsby.m NOT Found - Bandpass Filter Disabled'; 
else  Formats(8,1).enable  = 'on'; disp 'ormsby.m Found' ;  
end
%
Prompt(14,:)           =  {'Signal SPIKES Filter: ','SPKfilt',' If Amplitude is > X times the adjacents, is replaced by adjacents mean.'};
Formats(9,1).type      =  'list';
Formats(9,1).style     =  'popupmenu';
Formats(9,1).items     =  { 'NO','2','3','4','5','6','7','15','30','AUTO'};
DefAns(1).SPKfilt      =  Answer.SPKfilt;
%   
Prompt(15,:)      =  {' Resampling Factor: ','Resamp',''};
Formats(10,1).type   =  'list';
Formats(10,1).style  =  'popupmenu';
Formats(10,1).items  =  { 'NO','2','3','4'};
DefAns(1).Resamp     =  Answer.Resamp;
%
Prompt(16,:)        =  {'Smooth navigation:  ', 'SmoothFactor','Number of values before & after to average each position (integer 0 - 50)'};
Formats(11,1).type     =  'edit';
Formats(11,1).format   =  'float';
Formats(11,1).size     =  40;
Formats(11,1).limits   =  [0 50]; 
DefAns(1).SmoothFactor =  Answer.SmoothFactor;
%
nota =  'in Minutes (Integer 0 - 20) 0: NO split files';
Prompt(17,:)       =  {'SPLIT files if Time Gap between shots exceeds ', 'Maxgap',nota};
Formats(12,1).type    =  'edit';
Formats(12,1).format  =  'integer';
Formats(12,1).size    =  40;
Formats(12,1).limits  =  [0 20]; 
DefAns(1).Maxgap      =  Answer.Maxgap;
% 
Prompt(18,:)       =  {'FILTER "ON STATION" FILES: Min. Mean speed   ' ,'ONstation',' 0 Knots =  No filter'};
Formats(13,1).type    =  'edit';
Formats(13,1).format  =  'float';
Formats(13,1).size    =  40;
Formats(13,1).limits  =  [0 15];
DefAns(1).ONstation   =  Answer.ONstation;

% --------------------------------------- OFFSETS and CMP CALCULATION ---------------------------',[],[]};

Memo  =  ['        For Shalow water towed sources or CMP calculation & few channels systems.             '...   
          '        SourceX & Y are recalculated, Original souce fields are copied to GroupX &Y fields.  '...
          '        SEG-Ys  FIELD-RECORD & TRACE-NUMBER headers SORTED IS NECESSARY                     '... 
          '        0ffsets will be written in byte 37-40 Offset-TraceHeader.                            '...                
          '        new calculated positions will be written in cdpX & cdpY headers                       '...
          '        Fold = 0.5 *((inter Channel distance * Channels) / (2 * mean(shot interval)            '];

Prompt(19,:)         =  {'Offsets Info','',''};
Formats(13,2).type      =  'button';
Formats(13,2).size      =  100;
Formats(13,2).callback  =  @(~,~,handles,k)msgbox(sprintf(Memo,get(handles(k),'String')),'modal');
%
Prompt(20,:)         =  {'Apply LAYBACK (1 channel) or GEOMETRY  ( for Shot/Channel Sorted Input File)' ,'cdp',''};
Formats(14,1).type      =  'check';
DefAns(1).cdp           =   false;
%
Prompt(21,:)       =  {' Distance GPS Antenna - Source        ' ,'GPSsour','meters'};
Formats(14,2).type    =  'edit';
Formats(14,2).format  =  'float';
Formats(14,2).size    =  40;
Formats(14,2).limits  =  [0 9999];
DefAns(1).GPSsour     =   Answer.GPSsour;
%
Prompt(22,:)       =  {' Number of Channels: ' ,'CHANS','[1 - 48]'};
Formats(15,1).type    =  'edit';
Formats(15,1).format  =  'integer';
Formats(15,1).size    =  40;
Formats(15,1).limits  =  [1 48];
DefAns(1).CHANS       =  Answer.CHANS;
%
Prompt(23,:)       =  {' Distance GPS Antenna - 1st Channel ' ,'GPS1ch','meters '};
Formats(15,2).type    =  'edit';
Formats(15,2).format  =  'float';
Formats(15,2).size    =  40;
Formats(15,2).limits  =  [0 9999];
DefAns(1).GPS1ch      =  Answer.GPS1ch ;
%
Prompt(24,:)  =  {' Normal MoveOut and/or Brute Stack) :  ','nmo',[]};
Formats(16,1).type   =  'list';
Formats(16,1).style  =  'popupmenu';
Formats(16,1).items  =  {'None', 'Normal Move Out','UNDO NMO ','NMO & CMP STACK', 'Common Mid Point STACK','Common Shot STACK' };
DefAns(1).nmo     =  1;
A  =  exist('cvnmo','file');
if A   ~=  2,     Formats(16,1).enable  = 'off'; else  Formats(16,1).enable  = 'on';end
%
Prompt(25,:)        =  {' Distance Between Channels              ' ,'InterCH','meters'};
Formats(16,2).type     =  'edit';
Formats(16,2).format   =  'float';
Formats(16,2).size     =  40;
Formats(16,2).limits   =  [0 9999];
DefAns(1).InterCH      =  Answer.InterCH;
%
Prompt(26,:)       =  {'Sound Velocity for NMO ', 'SoundSp', ' m/seg '};
Formats(17,1).type    =  'edit';
Formats(17,1).format  =  'float';
Formats(17,1).size    =  40;
Formats(17,1).limits  =  [1400 1600];
DefAns(1).SoundSp     =  1500 ;
A  =  exist('cvnmo','file');
if A   ~=  2,   Formats(17,1).enable  = 'off'; else  Formats(17,1).enable  = 'on';end

[Answer,Cancelled]  =  inputsdlg(Prompt,Title,Formats,DefAns,Options);

if Cancelled  ==  1, return; end
if Cancelled  ==  2
        Answer.DataFolder = pwd;
        Answer.Survey = 'Survey';
        Answer.CoorFormat = 1;                    Answer.UTMinp = 30;
        Answer.Hemis = 1;                         Answer.UTMout = 30;
        Answer.FilesEX = 1;                       Answer.FilesFor = 1;
        Answer.Envel = 0;                         Answer.SPKfilt = 1;
        Answer.Resamp = 1;                        Answer.SmoothFactor = 0;
        Answer.Maxgap = 0;                        Answer.ONstation = 0;
        Answer.cdp = false;                       Answer.notch = 1;
        Answer.BPfilter = {10 20 500 600 false};  Answer.CHANS = 1;  
        Answer.GPSsour = 0;                       Answer.GPS1ch = 0;
        Answer.InterCH = 0;                       Answer.SoundSp = 1500;
        savefile = 'S4K9def.mat'; save(savefile, 'Answer','-mat', '-v7'); 
        return;
end
savefile = 'S4K9def.mat'; save(savefile, 'Answer','-mat', '-v7'); % Save defaults

%% Variables asignations

switch  Answer.FilesFor
        case 1,  DSF_In  =  1;  case 2,  DSF_In  =  3; case 3,  DSF_In  =  5;
end;    REV_In = 1 ;REV_Out = 1; DSF_Out = 1; 

switch  Answer.FilesEX
        case 1, ext  =  '*.seg';  case 2, ext  =  '*.sgy';  case 3, ext  =  '*.segy';  case 4, ext  =  '*.tra';    
end;    ext=char(ext);

switch  Answer.CoorFormat
        case 1, COORDENADAS  =  1;  case 2, COORDENADAS  =  0; case 3, COORDENADAS  =  -2; case 4, COORDENADAS  =  -1;
        case 5, COORDENADAS  =  -9; case 6, COORDENADAS  =  2; case 7, COORDENADAS  =  3;  case 8, COORDENADAS  =  4';        
end;   ORICOOR  = COORDENADAS;      

zoneF  =  Answer.UTMinp; zoneT  =  Answer.UTMout; hemis  =  Answer.Hemis; minex  =  Answer.ONstation;  
DSPK   =  Answer.SPKfilt;
switch DSPK, case 8  ,DSPK =  15; case 9  ,DSPK =  30;  case 10 ,DSPK =  999;  end   

RESAMPLE_FACTOR  =  Answer.Resamp;  instamp  =  Answer.Envel ;

switch instamp,  case 1, instamp  =  'Y';  case 0, instamp  =  'N';  end; 
smooth  =   Answer.SmoothFactor;

GAP_T  =    Answer.Maxgap ; 
survey  =   Answer.Survey;
notch   =   Answer.notch;


bpfilter = cell2mat(Answer.BPfilter(1:4));
BPfilter = int32(cell2mat(Answer.BPfilter(5)));
F1 = bpfilter(1);  F2 = bpfilter(2);
F3 = bpfilter(3);  F4 = bpfilter(4);

cdp     =   Answer.cdp;

switch Answer.nmo, case 1, nmo = 0; case 2, nmo = 1; case 3, nmo = -1;case 4,nmo = 1;case 5,nmo=0;case 6,nmo=0;  end; 
if Answer.nmo==4 ||Answer.nmo==5,  STack= 1; else STack = 0;end
if Answer.nmo==6, STCH = 1; else STCH =0; end
if cdp == 1
    towed(1)  =  Answer.GPSsour;
    towed(2)  =  Answer.GPS1ch;
    if towed(2) == 0, towed(2) = towed(1); end
    for i= 2:1:Answer.CHANS
        towed(i+1)  =   towed(i)+ Answer.InterCH;  
    end
end    

%% Folders and Files

cd (Answer.DataFolder);
files   =  dir(ext);
fin     =  size (files,1);
mkdir('./KS_OK'); mkdir('./Unknown_errors');mkdir('./Position_errors');
mkdir('./stemp');mkdir('./OnStation_files');
kmlName  = ['./KS_OK/' survey '.kml'];
kmlfile  =  kmlName ; fid9  =  fopen(kmlfile, 'w');  % Inicializa kml
fprintf(fid9,'%s\r\n','<?xml version = "1.0" encoding = "UTF-8"?>');
fprintf(fid9,'%s\r\n','<kml xmlns = "http://earth.google.com/kml/2.1">');
fprintf(fid9,'%s\r\n','<Document>');

pwd;
while fin  ==  0; 
    nota1= 'ERROR:         NO "'; nota2= '" Files found in ';
    str  =  [nota1, ext, nota2, pwd]; disp (str);  break;
end  
disp('  NEW CREATED FILES will have the same name in a new subfolder : KS_OK ')


%% Splitting

if GAP_T > 0
    for k = 1:fin     
        infile = files(k).name; outfile = infile; [pathstr, name, ex]  =  fileparts(outfile); 
% <==  
        [data,SegyTraceHeaders,SegyHeader] = ReadSegy(infile,'revision',REV_In,'dsf',DSF_In);
% <== 
        tr = size(data,2); 
        dia = [SegyTraceHeaders.DayOfYear];hora = [SegyTraceHeaders.HourOfDay];mins = [SegyTraceHeaders.MinuteOfHour];
        ttm = (dia*24*60)+(hora*60)+mins;  temps(1) = 0;
        for i =  2:tr % calc. T and D partials
                   temps(i) = abs(ttm(i)-ttm(i-1));     %#ok<*SAGROW>
        end          
        tzini = 1;tzfin = 1;sgm = 1;
        for j = 2:tr     % find GAPS
            if temps(j) >=  GAP_T
               tzfin = j-1;     Newdata  =  data(:,tzini:tzfin);
               NewSegyTraceHeaders  =  SegyTraceHeaders(tzini:tzfin);
               NewSegyHeader = SegyHeader;   trozo  =  int2str(sgm);
               NewName = ['./stemp/' name '_S' trozo ex];
%  == >
               WriteSegyStructure(NewName,NewSegyHeader,NewSegyTraceHeaders,Newdata,'revision',REV_In,'dsf',DSF_In);   
%  == >
               tzini = j; sgm = sgm+1;
            else tzfin = j; trozo  =  int2str(sgm);
           end;
           if j == tr   
               Newdata  =  data(:,tzini:tzfin);
               NewSegyTraceHeaders  =  SegyTraceHeaders(tzini:tzfin);
               NewSegyHeader = SegyHeader;   trozo  =  int2str(sgm);
               if sgm ==  1; trozo  =  '0';end; 
               NewName = [pathstr './stemp/' name '_S' trozo ex1];
%  == > 
               WriteSegyStructure(NewName,NewSegyHeader,NewSegyTraceHeaders,Newdata,'revision',REV_In,'dsf',DSF_In);
%  == >
           end;
        end; 
        clear data; 
        cartel  =  [name '------------> SEGMENTS: ' int2str(sgm)];
        disp (cartel);
    end;
     como = ['*_S' ext];    prev =  './' ;
else como = ['../' ext];    prev =  '../';
end
chdir ('stemp');pwd;files = dir (como);fin = size (files,1);
figure

%% Main

for k = 1:fin
     
    COORDENADAS = ORICOOR;
    infile2 = files(k).name;  infile  =  [prev, infile2];
   
% <== 
    [data,SegyTraceHeaders,SegyHeader] = ReadSegy(infile,'revision',REV_In,'dsf',DSF_In);    
% <== 
    noticia  =  ' **********    Processing Line:  '; 
    str  =  [noticia, infile '  **********  '];  disp ' ' ;disp (str);   
    tr = size(data,2); sam  =  size(data,1); 
    %Total Time for actual line  
    dia = [SegyTraceHeaders.DayOfYear];        hora = [SegyTraceHeaders.HourOfDay];
	mins = [SegyTraceHeaders.MinuteOfHour];    segs = [SegyTraceHeaders.SecondOfMinute];
    diaf = dia(tr);          diai = dia(1);
    if diaf < diai, diaf = diai+1; end;
    horaf = hora(tr);      horai = hora(1);
    minutosf = mins(tr);   minutosi = mins(1);
    segundosf = segs(tr);  segundosi = segs(1);
    tiempoi = (diai*24)+horai+(minutosi/60)+(segundosi/3600);
    tiempof = (diaf*24)+horaf+(minutosf/60)+(segundosf/3600);
    ttrans(k) = tiempof-tiempoi; 

 %% notch filtering with transient state suppression).
    
 if notch > 1 
        
            switch notch
                case  2,  FD = 50;
                case  3,  FD = 60;
                case  4,  FD = F1;
            end  % frequency to filter in Hz.
             
            M = 15;  BW = 3;  % default  Samples 15 & Bandwidth 3 , more samples slower
             FS = 1000000/SegyHeader.dt; 
             str  =  [ num2str(FD), 'Hz Notch Filtering ...']; disp ' '; disp (str);
            
             for i= 1:1:tr
                traza  = data(:,i)' ;
                filtraza = ecgTransientSuppression(traza, FS, FD, BW, M);
                data(:,i)=filtraza';
            end
 end 
 
  %% BandPass filtering 
    
  %if  BPfilter == 1 &&  Formats(8,1).enable == 'on';     %#ok<*STCMP>
  if  BPfilter == 1; 
                FS = 1000000/SegyHeader.dt;                                % Sampling Frequency (Hz)
                fs = SegyHeader.dt/1000;                                   % Sampling Frequency(from microsec to miliSec)
                Fn = FS/2;                                                 % Nyquist Frequency (Hz)
                
                if F4 >= Fn,
                      str  =  [ mat2str(Fn) 'Hz Nyquist Freq.']; disp ' '; disp (str);     
                      disp ' Frequencies to High' ;
                      F4= Fn - 1; 
                      if F3 >= F4 , F3 = F4 - 50; end
                      str  =  [ mat2str(F1), ' ', mat2str(F2),' ', mat2str(F3),' ',...
                       mat2str(F4),' Hz BandPass Filtering ...']; disp ' '; disp (str);  
                 else  
                      str  =  [ mat2str(bpfilter) ' Hz BandPass Filtering ...']; disp ' '; disp (str);
                end
                 
                data = ormsby( data ,fs ,F1 ,F2 ,F3 ,F4);           
  end
    
 %%  spikes removal (conditional)--------------------------------------
    
    if DSPK > 1 ;
        disp ' '; disp (' cleaning spikes ...');        
        
        % statistics     
       media  =   median(data);         sd  =  std(data);
       gr     =   media +  (0.6 * sd); gr = mean(gr);            
       pt     =   media -  (0.6 * sd); pt = mean(pt);
       dato = data; 
       for c  =  2:1:tr-1; % para cada columna
            if  SegyTraceHeaders(c).DelayRecordingTime  ==  SegyTraceHeaders(c-1).DelayRecordingTime && ...
                SegyTraceHeaders(c).DelayRecordingTime  ==  SegyTraceHeaders(c+1).DelayRecordingTime;
                if DSPK == 999                    % ===== AUTO  
                    for f = 1:1:sam;                         
                         if data(f,c) > gr || data(f,c) < pt
                                dato(f,c)  =  0.5 * (data(f,c-1) + data(f,c+1));                                 
                         else
                                dato(f,c) =  data(f,c);
                         end
                    end        
                else                               % ====== MANUAL
                    for f = 1:1:sam;  % para cada file         
                            dato(f, c)  = data(f,c); 
                            if abs(data(f, c)) >=  DSPK * abs(data(f,c-1)) && abs(data(f,c)) >=  DSPK * abs(data(f,c+1)) 
                                dato(f, c)  =  0.5 * (data(f,c-1) + data(f,c+1)); 
                            end;                        
                    end; %for f
                end %if else DSPK
            end; %if
        end; %for c
        data = dato;       clear dato;
    end; 
   
  %% Inst. Ampl. Envelop --(conditional)------------------------------------------------
   
    if  instamp  ==  'Y'; 
            datenv = zeros(sam,size(data,2));
            for tnum = 1:1:size(data,2)
                tracex = data(:,tnum)'; tracex = tracex/max(tracex);
                data(:,tnum) = 20 * log10(abs(myhilbert(tracex + eps)));
            end;
            disp ' '; disp (' Inst. Ampl. Envelope applied...');
    end;
    
 %% Data Resampling  (always but may no change if R_F 1)-----------------------------
 
    if RESAMPLE_FACTOR  ==  0, RESAMPLE_FACTOR  =  1; end;
    if RESAMPLE_FACTOR > 1, disp ' '; disp (' Resampling...'); end
    Rdata = data(1:RESAMPLE_FACTOR:end,:);
    SegyHeader.dt =  floor(SegyHeader.dt*RESAMPLE_FACTOR); 
    SegyHeader.ns = size(Rdata,1);sam = size(Rdata,1);
    SegyHeader.dtOrig =  SegyHeader.dt; SegyHeader.nsOrig = SegyHeader.ns;
    punt = 0;
    clear data;
     
    %% Coordinates headers  Reformat  & if necessary transform from UTM to Geographic for change of fuse--  
 
    switch COORDENADAS
    case  3   % format DDDMMMMM 
        Dlon = fix([SegyTraceHeaders.SourceX]/100000);
        Dlat = fix([SegyTraceHeaders.SourceY]/100000);
        Mlon = (([SegyTraceHeaders.SourceX]/100000)-Dlon)*100;
        Mlat = (([SegyTraceHeaders.SourceY]/100000)-Dlat)*100;
        Lon  =  Dlon + (Mlon/60);
        Lat  =  Dlat + (Mlat/60);
    case  2  %format  DDMMM  ( to interpolate)
        Dlon = fix([SegyTraceHeaders.SourceX]/1000);
        Dlat = fix([SegyTraceHeaders.SourceY]/1000);
        Mlon = (([SegyTraceHeaders.SourceX]/1000)-Dlon)*100;
        Mlat = (([SegyTraceHeaders.SourceY]/1000)-Dlat)*100;
        Lon  =  Dlon + (Mlon/60);    
        Lat  =  Dlat + (Mlat/60);
        x = double(Lon);  
        y = double(Lat);
        cdpX = x;   
        cdpY = y; %( pre interpolated coordinates )
        % polynomial fit
        difx = abs(max(x)-min(x));
        dify = abs(max(y)-min(y));
        ctx = 0;cty = 0;
        for i = 2 :tr ; 
           if x(i)  ~=  x(i-1); ctx  =  ctx+1;end;
           if y(i)  ~=  y(i-1); cty  =  cty+1;end;
        end
        if tr >=  20 
            if difx >=  dify && ctx >=  10
                [coef,S] = polyfit(x,y,5);
                Lon = linspace(x(1),x(tr), tr);
                Lat = polyval(coef,Lon); 
            end
            if dify > difx && cty >=  10     
                [coef,S] = polyfit(y,x,4);  
                Lat = linspace(y(1),y(tr), tr);
                Lon = polyval(coef,Lat);
            end
            if difx == dify && difx  ==  0 || ctx <10 && cty < 10
                Lon = x ;Lat = y; punt  = 1; 
            end
        end
        ctx = 0;cty = 0;
    case 1           %Degrees*3600000 (miliseconds of arc) 
        Lon = [SegyTraceHeaders.SourceX]/3600000;    
        Lat = [SegyTraceHeaders.SourceY]/3600000;
    case 4            %Degrees*360000 (centiseconds of arc) 
        Lon = [SegyTraceHeaders.SourceX]/360000;    
        Lat = [SegyTraceHeaders.SourceY]/360000;    
    otherwise  % metric system
        x = [SegyTraceHeaders.SourceX]; y = [SegyTraceHeaders.SourceY];
        if COORDENADAS  ==  -1; x = round(x/10); y = round(y/10);   end
        if COORDENADAS  ==  -2; x = round(x/100); y = round(y/100); end   
        if COORDENADAS  ==  -9; x = [SegyTraceHeaders.GroupX]; y = [SegyTraceHeaders.GroupY]; end
        Lon = x;  Lat = y;     %para el mapa
        if zoneF ~=  zoneT  % change UTM fuse 
            disp ' '; disp (' Changing UTM fuse ...');
            COORDENADAS = 9; % 
            [ Lon, Lat ]  =  UTMGeo(x,y,zoneF,hemis );
        end 
    end %switch
    
%% discrimine some known errors on segy navigation data --------------------

    nav_error  =  0;       LonMax =  max(Lon);     LonMin = min(Lon);  
    LatMax = max(Lat);     LatMin = min(Lat);
    if  COORDENADAS < 1   %metric
        if  LonMax == 0 || LonMin == 0 || LatMax == 0 || LatMin == 0;nav_error  =  1;end 
    end
    if  COORDENADAS > 0   %geo
        if LatMin<-90 || LatMax>90 || LonMin<-180 || LonMax>360; nav_error  =  1;end
    end
    
%% Create MAP (Always)

    B = exist('geoshow','file');
    [path, nom, ex]  =  fileparts(files(k).name);
    kn  = ['\fontsize{8} ' nom];
    ccc = -ccc;            % altenate colors red-green
    if ccc == 1,color = 'r' ; else color  =  'g'; end
    if B > 0
       
        if nav_error  ==  0 && punt  ==  0
            if Lon(1) < 360 
                gmap =  geoshow (Lat, Lon);      %geoshow
            else
              [Lon, Lat]  =  UTMGeo(x,y,zoneF,hemis ); gmap =  geoshow (Lat, Lon);
            end           
            text(Lon(1),Lat(1), kn); set(gmap ,'Color',color,'LineWidth',2);
            if k == fin; 
                grid on; title('Labels on line start)');    
                saveas(gcf,'../KS_OK/MAP.jpg','jpg');   
            end    
        end 
    end
    
 %% if necessary trans GEO to UTM -----------------------------------

 x = zeros(tr,1); y = zeros(tr,1);  tr = length(Lat); %#ok<*NASGU>
    if COORDENADAS  == 2  % only for preinterpolated coordinates
       [x,y] =  GeoUTM (cdpX,cdpY,zoneT);
       for i = 1:length(x)
            SegyTraceHeaders(i).cdpX  =  round(x(i)); 
            SegyTraceHeaders(i).cdpY  =  round(y(i)); 
       end
    end
    if COORDENADAS >=  1  % all GEO?
        disp ' '; disp (' Lat-Lon to UTM');
        [x,y] =  GeoUTM (Lon,Lat,zoneT);
        for i = 1: length(x) 
            if ORICOOR > -8
                SegyTraceHeaders(i).GroupX  =  SegyTraceHeaders(i).SourceX; 
                SegyTraceHeaders(i).GroupY  =  SegyTraceHeaders(i).SourceY; 
            end
            SegyTraceHeaders(i).SourceX = round(x(i));                  
            SegyTraceHeaders(i).SourceY = round(y(i));   
        end; 
    end;     
        
%% Total distance & speed in line (in nautic miles)
 
x = [SegyTraceHeaders.SourceX];   
    y = [SegyTraceHeaders.SourceY];
    LonMax =  x(tr);  LonMin  = x(1);   LatMax =  y(tr);   LatMin = y(1);
    distt(k) = sqrt(double(((LatMax-LatMin)^2)+((LonMax-LonMin)^2)))/1852; %nm
    MeanSpeed(k) = single(distt(k)/ttrans(k));    % in knots
    noticia  =  ' Mean Speed for line ) ';
    str  =  [noticia, infile '  =  ', num2str(MeanSpeed(k)), ' knots'];
    disp ' ';disp (str);

 %% smoothing -----Moving average 1 + 2*smooth shots/pings ---(conditional)-------------

    if smooth > 0 
          disp ' '; disp ('Smoothing... ');
          N = 1 + (2*smooth);  
          for i = 1 :1:tr    % store pre-smooth in groupX&Y
            SegyTraceHeaders(i).GroupX  =  SegyTraceHeaders(i).SourceX; 
            SegyTraceHeaders(i).GroupY  =  SegyTraceHeaders(i).SourceY;
          end                 
          for i = smooth+1:1:tr-smooth  
              j = i-smooth;g = i+smooth;
              mmvx(1:N)  =  SegyTraceHeaders(j:g).GroupX ;
              mmvy(1:N)  =  SegyTraceHeaders(j:g).GroupY ;
              SegyTraceHeaders(i).SourceX  =  int32(sum(mmvx)/N);                     
              SegyTraceHeaders(i).SourceY  =  int32(sum(mmvy)/N);
         end;
    end  
 
 %% position calculations for shallow water towed sources OR multichannel offsets---(conditional)--------
 
    if cdp == 1
        if Answer.CHANS == 1;    % SINGLE CHANNEL---------------------
                if towed(2) == 0 , towed(2) = towed(1);end  
                disp ' '; disp ('Applying Single Channel Offset... ');      
                for i = 2:1:tr         
                        ITX(i) = SegyTraceHeaders(i).SourceX - SegyTraceHeaders(i-1).SourceX ;  %Incremet X
                        ITY(i) = SegyTraceHeaders(i).SourceY - SegyTraceHeaders(i-1).SourceY ;  %id Y
                        ITXY(i) = round(sqrt (double(ITX(i)^2+ITY(i)^2))) ; % distance to previous ping/shot
                end
                ITX(1) = 0; ITY(1) = 0; ITXY(1) = 0;ofo=zeros(tr);
                SourceX =zeros(tr,1);  SourceY=zeros(tr,1);     
                for i = 1:1:tr    
                        SegyTraceHeaders(i).offset  =  towed(1) + ( ( towed(2)- towed(1) ) /2 );
                        g  =  i ;  SUMA  =  ITXY(i);
                        while g > 1
                            if SUMA >=  towed(1), break ,end
                            g  =  g - 1; SUMA  =  SUMA + ITXY(g);
                        end
                        if g > 1
                            SourceX(i)  =  SegyTraceHeaders(g-1).SourceX;
                            SourceY(i)  =  SegyTraceHeaders(g-1).SourceY;
                            ofo(i)  =  i - g;
                        else
                            SourceX(1)  =  SegyTraceHeaders(1).SourceX;
                            SourceY(1)  =  SegyTraceHeaders(1).SourceY;   
                        end
                end
                for i = 1:1:tr
                   if ofo(i)>0, g = i-1;   break;  end  
                end
                CITX = sum(ITX(1:g));  CITY = sum(ITY(1:g));
                for i = 1:1:g           
                        SegyTraceHeaders(i).cdpX   =    SegyTraceHeaders(i).SourceX - CITX;
                        SegyTraceHeaders(i).cdpY    =   SegyTraceHeaders(i).SourceY - CITY;
                end
                
        else              % MULTICHANNEL  ------------------

                disp ' '; disp (' Applying Multichannel Geometry ... ');
                channels  =  Answer.CHANS;
                for ch = 1:1:channels
                    ChOff(ch)  =  towed(1) + ((towed(ch+1)-towed(1))/2); 
                end; 
                for i =  1:1:tr
                    sh(i)   =   SegyTraceHeaders(i).FieldRecord; %FFID
                    ch(i)   =   SegyTraceHeaders(i).TraceNumber; 
                    SegyTraceHeaders(i).offset   =   int32(ChOff(ch(i)));
                end
                
                for i=1:channels, ITX(i) = 0; ITY(i) = 0; ITXY(i) = 0; end
                for i = channels+1:1:tr
                    if sh(i) > sh(i-1) 
                        ITX(i)  =  SegyTraceHeaders(i).SourceX - SegyTraceHeaders(i-1).SourceX ;
                        ITY(i)  =  SegyTraceHeaders(i).SourceY - SegyTraceHeaders(i-1).SourceY ;
                        ITXY(i) = (sqrt(double(ITX(i)^2+ITY(i)^2))) ; % dist to previous shot
                    else
                        ITX(i)  =  ITX(i-1);  
                        ITY(i)  = ITY(i-1);  
                        ITXY(i) = ITXY(i-1);
                   end
                end
                for i = 1:channels
                   ITX(i)  = ITX(channels+i); 
                   ITY(i)  = ITY(channels+i);  
                   ITXY(i) = ITXY(channels+i);
                end
                
                for i = 1:1:tr
                    itx(i)  =   ChOff(ch(i))*  (ITX(i)  / ITXY(i));
                    ity(i)  =   ChOff(ch(i))*  (ITY(i) /  ITXY(i));
                end
                
                sourceX =zeros(tr,1);        sourceY=zeros(tr,1);         SNav= zeros(tr,6);
                
                for i = 1:1:tr
                    
                    SegyTraceHeaders(i).cdpX   =  SegyTraceHeaders(i).SourceX - itx(i);
                    SegyTraceHeaders(i).cdpY   =  SegyTraceHeaders(i).SourceY - ity(i);
                    sourceX(i)   =  SegyTraceHeaders(i).cdpX;
                    sourceY(i)   =  SegyTraceHeaders(i).cdpY;
                    SNav(i,1) = int32(sourceX(i));
                    SNav(i,2) = int32(sourceY(i));
                    SNav(i,3) = sh(i);
                    SNav(i,4) = ch(i);
                    SNav(i,6) = SegyTraceHeaders(i).offset;
                    
                    
                end           
                Fold = 0.5 *((Answer.InterCH * Answer.CHANS) / (2 * mean(ITXY))); 
                difX  =  SNav(1,1) -  SNav(tr,1);
                difY  =  SNav(1,2) -  SNav(tr,2);
                if abs(difX) >= abs(difY) 
                    if difX < 0, sf = 1;  else sf = -1;  end
                else
                    if difY < 0; sf = 2; else sf = -2; end
                end
                SNav = sortrows(SNav,sf);
                SNav(1,5) = 1;
                lx = SNav(1,1) + (sign(sf)* Fold);
                ly = SNav(1,2) + (sign(sf)* Fold);
                for i = 2:1:tr
                    if sign(sf) == -1
                            if abs(sf)== 2 
                                if SNav(i,2) >= ly %sign -
                                  SNav(i,5) = SNav(i-1,5) ;
                                else
                                  ly = SNav(i,2) + (sign(sf) * Fold);
                                  SNav(i,5) = SNav(i-1,5) + 1;
                                end
                            else   
                                if SNav(i,1) >= lx  %sign -
                                  SNav(i,5) = SNav(i-1,5) ;
                                else
                                  lx = SNav(i,1)+(sign(sf)* Fold);
                                  SNav(i,5) = SNav(i-1,5)+1;
                                end
                            end %if abs..
                    else %sign == 1
                            if abs(sf)== 2 
                                if SNav(i,2) <= ly %sign -
                                  SNav(i,5) = SNav(i-1,5) ;
                                else
                                  ly = SNav(i,2) + (sign(sf) * Fold);
                                  SNav(i,5) = SNav(i-1,5) + 1;
                                end
                            else   
                                if SNav(i,1) <= lx  %sign -
                                  SNav(i,5) = SNav(i-1,5) ;
                                else
                                  lx = SNav(i,1)+(sign(sf)* Fold);
                                  SNav(i,5) = SNav(i-1,5)+1;
                                end
                            end %if abs..
                           
                    end % sign 
                end %tr
                SNav2 = sortrows(SNav,[3,4]);
                for i= 1:1:tr
                    SegyTraceHeaders(i).cdp = SNav2(i,5);
                end
                clear SNav SNav2;
        end %cdp
     
   end % offset  
     
%% NMO (conditional)-------------------------------
if nmo ~= 0;  
       disp ' '; disp (' Applying NMO Corrections ... ');
        NMOdata = zeros(sam,tr); 
        for i= 1:1:tr    
             spd   =   Answer.SoundSp; 
             dt    =   SegyHeader.dt / 1000000;  
             off   =   SegyTraceHeaders(i).offset;
             D_in  =   Rdata(:,i);
             [D_out] = cvnmo( D_in , double(off), double(dt), double(spd) ,nmo);
             % cvnmo SeismicLab Function
             NMOdata(:,i) =  D_out;
        end
        Rdata = NMOdata;   clear   NMOdata
end

%% channels stack for each shot

if STCH == 1    % channels of shot stack
     disp ' '; disp (' Applying Channels Stack ... ');
     chans = Answer.CHANS;
     tr = size(Rdata,2);
     shts=int32(tr/chans);
     for i=1:1:shts, SegyTraceHeaderS = SegyTraceHeaders; end
     Kdata =  zeros(size(Rdata,1),shts);
     data = Rdata(:,1); ac = 1;
     for i= 2:1:tr
            if SegyTraceHeaders(i).FieldRecord == SegyTraceHeaders(i-1).FieldRecord 
                data = data + Rdata(:,i);
                ac= ac + 1;
                shot =  SegyTraceHeaders(i).FieldRecord;
            else
                Kdata(:,shot) = data/ac;
                SegyTraceHeaderS(shot).TraceSequenceLine = shot;
                SegyTraceHeaderS(shot).SourceX = SegyTraceHeaders(i-1).SourceX;
                SegyTraceHeaderS(shot).SourceY = SegyTraceHeaders(i-1).SourceY;
                SegyTraceHeaderS(shot).LagTimeA  = SegyTraceHeaders(i-1).LagTimeA ; 
                SegyTraceHeaderS(shot).DelayRecordingTime =  SegyTraceHeaders(i-1).DelayRecordingTime;
                SegyTraceHeaderS(shot).NStackedTraces=ac; 
                shot = SegyTraceHeaders(i).FieldRecord;              
                data = Rdata(:,i);
                ac = 1;
            end
     end
end
%% CDP Sorting & Stacking (conditional)-------------------------

if STack == 1
        disp ' '; disp (' Applying Brute Stack ... ');
        % sorting by CDP
        tr = size(Rdata,2); 
        SeisTab = zeros(tr,4);
        SegyTraceHeaderS = SegyTraceHeaders;
        for i =1:1:tr  
            SeisTab(i,1)=  i;
            SeisTab(i,2) = SegyTraceHeaders(i).cdpX;
            SeisTab(i,3) = SegyTraceHeaders(i).cdpY;
            SeisTab(i,4) = SegyTraceHeaders(i).cdp + 1; % avoid cdp 0
        end
        SortTab  =  sortrows(SeisTab,4);
        Sdata =  zeros(size(Rdata));
        %NA = 0;
        for i= 1:1:tr
            ord = int32(SortTab(i,1));
            Sdata(:,i) =  Rdata(:,ord);  % Sdata: data sorted by column 4 (cdp)
            SegyTraceHeaderS(i).TraceSequenceLine = i;
            SegyTraceHeaderS(i).SourceX = SortTab(i,2);
            SegyTraceHeaderS(i).SourceY = SortTab(i,3);
            SegyTraceHeaderS(i).offset  = 0;
            SegyTraceHeaderS(i).cdp     = SortTab(i,4);
            SegyTraceHeaderS(i).GroupX  = 0; 
            SegyTraceHeaderS(i).GroupY  = 0; 
            SegyTraceHeaderS(i).cdpX  = 0; 
            SegyTraceHeaderS(i).cdpY  = 0;
            SegyTraceHeaderS(i).FieldRecord = 1; 
            SegyTraceHeaderS(i).TraceNumber = 1;
            SegyTraceHeaderS(i).EnergySourcePoint = 0;
            SegyTraceHeaderS(i).LagTimeA  = SegyTraceHeaders(ord).LagTimeA ; 
            SegyTraceHeaderS(i).DelayRecordingTime =  SegyTraceHeaders(ord).DelayRecordingTime;
            SegyTraceHeaderS(i).ns = SegyHeader.ns;
 	        SegyTraceHeaderS(i).dt = SegyHeader.dt;
        end
        
        CDPS = max(SortTab(:,4));
        Kdata=  zeros(size(Rdata,1),CDPS) ;
        
        data = Sdata(:,1); ac = 1;
        cmp  = SegyTraceHeaderS(1).cdp  ;
       
        for i= 2:1:tr
            
            if SegyTraceHeaderS(i).cdp == cmp
                data = data + Sdata(:,i);
                ac= ac + 1;           
            else               
               Kdata(:,cmp) = data/ac;
               SegyTraceHeaderS(cmp).TraceSequenceLine = cmp;
               SegyTraceHeaderS(cmp).SourceX = SortTab(i-1,2);
               SegyTraceHeaderS(cmp).SourceY = SortTab(i-1,3);
               SegyTraceHeaderS(cmp).cdp= SortTab(i-1,4);
               SegyTraceHeaderS(cmp).LagTimeA  = SegyTraceHeaders(i-1).LagTimeA ; 
               SegyTraceHeaderS(cmp).DelayRecordingTime =  SegyTraceHeaders(i-1).DelayRecordingTime;
               SegyTraceHeaderS(cmp).NStackedTraces=ac; 
               cmp = SegyTraceHeaderS(i).cdp;               
               data = Sdata(:,i);
               ac = 1;
            end          
        end    
end % STack

%%  File sorting ----ALWAYS------------------------------------------
   last = size(SegyTraceHeaders,2);
   TSL  =  GetSegyTraceHeaderInfo(SegyTraceHeaders(1,last),'TraceSequenceLine');    
   if (TSL >=  10000000)||(TSL  ==  0)  
        outfile = ['../Unknown_errors/' infile2];     % directorio erroneos
        disp ' ';disp (' Some unknow error in SEGY files, see corresponding folder');
   else 
       if nav_error == 0 
          if MeanSpeed(k) <=  minex && minex > 0 || LonMax == LonMin  && LatMin == LatMax 
            outfile = ['../OnStation_Files/'  infile2]; % "too slow" file
            disp;disp (' Some "On Station" files, see On Station folder');
          else   
            if RESAMPLE_FACTOR > 1, infile2            =  [ 'r_' infile2];     end; %#ok<*AGROW>
            if instamp == 'Y', infile2                 =  [ 'e_' infile2];     end;
            if DSPK     > 1, infile2                   =  [ 'f_' infile2];     end;
            if notch    > 1, infile2                   =  [ 'n_' infile2];     end;
            if smooth   > 0, infile2                   =  [ 's_' infile2];     end;
            if BPfilter == 1, infile2                  =  [ 'bp_' infile2];    end;
            if cdp      > 0 && Answer.CHANS == 1, infile2  =  [ 'o_' infile2];     end;
            if cdp      > 0 && Answer.CHANS >  1, infile2  =  [ 'cdp_' infile2];   end;
            if STack==1
                                           infile3 =  [ 'csk_' infile2]; 
                                          outfileST = ['../KS_OK/' infile3]; 
            end;
             if STCH==1
                                           infile3 =  ['ssk_' infile2]; 
                                          outfileST = ['../KS_OK/' infile3]; 
            end;           
            
            if (nmo  == 1 ) ,             infile2 =   ['+nmo_' infile2]; end;
            if (nmo  == -1 ) ,            infile2 =   ['-nmo_' infile2]; end;
            total  =  total + tr;
                                         outfile   =  ['../KS_OK/' infile2];  
          end   
      else   % nav_error == 1
          outfile = ['../Position_errors/' infile2]; % errores de navegacion
          disp ' ';disp (' Some Coordinates errors, see  subfolders');
      end; 
   end; 
 
%% -----------Write output file (SEGY & KML) ---------

    fprintf(fid9,'<Placemark>\r\n');
    fprintf(fid9,strcat('<name>',files(k).name,'</name>\r\n'));
    fprintf(fid9,'<ExtendedData>\r\n');   
    fprintf(fid9,strcat('<Data name = "Survey"><value>',survey,'</value></Data>\r\n'));
    fprintf(fid9,strcat('<Data name = "Segyfile"><value>',files(k).name,'</value></Data>\r\n'));
    fprintf(fid9,'<Data name = "Traces"><value>%7g</value></Data>\r\n',tr);
    fprintf(fid9,'<Data name = "Samples/trace"><value>%10f</value></Data>\r\n',SegyHeader.ns);
    fprintf(fid9,'<Data name = "Sample_Int."><value>%10f</value></Data>\r\n',SegyHeader.dt);
    fprintf(fid9,'<Data name = "Lon-S"><value>%-+10.5f</value></Data>\r\n',Lon(1));
    fprintf(fid9,'<Data name = "Lat-S"><value>%-+10.5f</value></Data>\r\n',Lat(1));
    fprintf(fid9,'<Data name = "Lon-E"><value>%-+10.5f</value></Data>\r\n',Lon(tr));
    fprintf(fid9,'<Data name = "Lat-E"><value>%-+10.5f</value></Data>\r\n',Lat(tr));
    fprintf(fid9,'<Data name = "Zone_UTM"><value>%g</value></Data>\r\n',zoneT);
    fprintf(fid9,'<Data name = "Speed (kn)"><value>%4.2f</value></Data>\r\n',MeanSpeed(k));
    fprintf(fid9,'<Data name = "Time Start"><value>%2g:%2g:%2g</value></Data>\r\n',...
        SegyTraceHeaders(1).HourOfDay,SegyTraceHeaders(1).MinuteOfHour,SegyTraceHeaders(1).SecondOfMinute );    
    fprintf(fid9,'<Data name = "Time End"><value>%2g:%2g:%2g</value></Data>\r\n',...
        SegyTraceHeaders(tr).HourOfDay,SegyTraceHeaders(tr).MinuteOfHour,SegyTraceHeaders(1).SecondOfMinute ); 
    fprintf(fid9,'<Data name = "Date (Y/JD)"><value>%4g/%3g </value></Data>\r\n',...
        SegyTraceHeaders(1).YearDataRecorded, SegyTraceHeaders(1).DayOfYear);  
    fprintf(fid9,'</ExtendedData>\r\n');
    fprintf(fid9,'<Point><coordinates>%g,%g,0</coordinates></Point>\r\n',Lon(1),Lat(1));
    fprintf(fid9,'</Placemark>\r\n');
    fprintf(fid9,'<Placemark>\r\n');
    fprintf(fid9,'<LineString>\r\n');
    fprintf(fid9,'<coordinates>\r\n');
    for i =  1:1:tr
        fprintf(fid9,'%g,%g,0\r\n',Lon(i),Lat(i));
        SegyTraceHeaders(i).TraceSequenceFile = i;
        SegyTraceHeaders(i).CoordinateUnits = 1;                 
        SegyTraceHeaders(i).SourceGroupScalar = 1; 
        SegyTraceHeaders(i).LagTimeA  =  SegyTraceHeaders(i).DelayRecordingTime;
        SegyTraceHeaders(i).dt =  SegyHeader.dt;                 
        SegyTraceHeaders(i).ns =  SegyHeader.ns;                      
        tops(i) = SegyTraceHeaders(i).LagTimeA;
    end;
    
%  == >  
    if STack==1 || STCH ==1
        WriteSegyStructure(outfileST,SegyHeader,SegyTraceHeaderS,Kdata,'revision',REV_Out,'dsf',DSF_Out);
    else  
        WriteSegyStructure(outfile,SegyHeader,SegyTraceHeaders,Rdata,'revision',REV_Out,'dsf',DSF_Out);
    end
%  == >
    fprintf(fid9,'</coordinates>\r\n');
    fprintf(fid9,'</LineString>\r\n');
    fprintf(fid9,'</Placemark>\r\n');
    orden(k) = k;
    ran(k) = int32(SegyHeader.dt * SegyHeader.ns / 1000);
    maxtop(k) = int32(max(tops)+ran(k));
    mintop(k) = int32(min(tops));  
   
end;  % MAIN

fprintf(fid9,'%s\r\n','</Document>');
fprintf(fid9,'%s\r\n','</kml>');    %close kml

%% ---------Remove empty folders ------------------------

fclose ('all');chdir ('..');rmdir('stemp','s')
if any(size(dir(['./Unknown_errors/' ext ]),1))   ==  0, rmdir('./Unknown_errors'); end
if any(size(dir(['./OnStation_files/' ext ]),1))  ==  0, rmdir('./OnStation_files');end
if any(size(dir(['./Position_errors/' ext ]),1))  ==  0, rmdir('./Position_errors');end

%-------------Ranges display --------------------------
if max(orden)> 1 
    figure(2); ccc = 1;
    ploteo  =  errorbar(orden,mintop,(ran-ran),maxtop-mintop,':kv','LineWidth',2);
    set(gca,'YDir','reverse');
    grid off;
    for i = 1:1:max(orden)
        ccc = -ccc;
        if ccc == 1,color = 'r' ; else color  =  'g'; end
        [path, nom, ex]  =  fileparts(files(i).name); 
        ks = ['\fontsize{10} ' nom]; nks  =  strrep(ks, '_', '\_');
        text(double(i)-0.2,double(mintop(i))+50,nks,'Rotation',270,'color',color);
    end
    ylabel('Minimum TimeDelay & Full Range in file','fontsize',11);
    xlabel('Map file number','fontsize',8);
    saveas(gcf,'./KS_OK/segy-ranges.jpg','jpg')
end
%--------------------------------------------------
 disp ' ';disp 'SEG4KING END';fclose all;beep;


