function [D_out] = cvnmo(D_in,offset,dt,v,conj);  
%  CVNMO: constant velocity NMO and NMO^{-1} correction 
%
%  Apply constant velocity NMO and inverse NMO 
%  correction to CMP gatherts
%
%  [D_out] = cvnmo(D_in,offset,dt,v,conj)
%
%  IN    D_in: data to NMO or un-NMO
%        v: velocity in m/sec 
%        offset: source-receicer distance in meters
%        dt: sampling in secs
%        conj: flag to    NMO conj=1
%        conj: flag to un-NMO conj=-1
%
%  OUT   D_out: output data after NMO or un-NMO
%
%
%
%  SeismicLab
%  Version 1
%
%  written by M.D.Sacchi, last modified December 10, 1998.
%  sacchi@phys.ualberta.ca 
% 
%  Copyright (C) 1998 Seismic Processing and Imaging Group 
%                     Department of Physics
%                     The University of Alberta 
%
%

[nt,nh]=size(D_in);
D_out=zeros(nt,nh);

n=1:1:nt;

for k=1:nh
 d=sqrt( ((n-1)*dt).^2+(offset(k)/v)^2);
 nn=floor(d/dt)+1; 
if conj == 1;   
  for kk=1:nt
   if nn(kk)<nt; D_out(n(kk),k) = D_in(nn(kk),k);end;
  end
  end
if conj ==-1;   
  for kk=1:nt
   if nn(kk)<nt; D_out(nn(kk),k) = D_in(n(kk),k);end;
  end
 end
 end


 
return
